package com.minerarcana.floralchemy;

import java.io.File;

import com.minerarcana.floralchemy.api.FloralchemyAPI;
import com.minerarcana.floralchemy.block.BlockCrystalthorn;
import com.minerarcana.floralchemy.block.BlockFloodedSoil;
import com.minerarcana.floralchemy.block.BlockHedge;
import com.minerarcana.floralchemy.block.BlockLeakyCauldron;
import com.minerarcana.floralchemy.loot.LootFunctionCrystalthorn;
import com.minerarcana.floralchemy.village.VillageHedgeHouse;
import com.minerarcana.floralchemy.village.VillageHedgedHouseHandler;
import com.teamacronymcoders.base.BaseModFoundation;
import com.teamacronymcoders.base.registrysystem.BlockRegistry;

import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.Tuple;
import net.minecraft.world.gen.structure.MapGenStructureIO;
import net.minecraft.world.storage.loot.LootTableList;
import net.minecraft.world.storage.loot.functions.LootFunctionManager;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.Mod.EventBusSubscriber;
import net.minecraftforge.fml.common.Mod.EventHandler;
import net.minecraftforge.fml.common.Mod.Instance;
import net.minecraftforge.fml.common.SidedProxy;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.registry.VillagerRegistry;

@Mod(modid = Floralchemy.MOD_ID, name = Floralchemy.MOD_NAME, version = Floralchemy.VERSION, dependencies = Floralchemy.DEPENDS)
@EventBusSubscriber
public class Floralchemy extends BaseModFoundation<Floralchemy> {

    public static final String MOD_ID = "floralchemy";
    public static final String MOD_NAME = "Floralchemy";
    public static final String VERSION = "1.12.2-1.1.0";
    public static final String DEPENDS = "required-after:base@[3.13.0,);after:thaumcraft; after:botania";

    @Instance("floralchemy")
    public static Floralchemy instance;

    @SidedProxy(clientSide = "com.minerarcana.floralchemy.ClientProxy", serverSide = "com.minerarcana.floralchemy.CommonProxy")
    public static CommonProxy proxy;

    public Floralchemy() {
        super(MOD_ID, MOD_NAME, VERSION, CreativeTabs.field_78026_f);
    }

    @Override
    @EventHandler
    public void preInit(FMLPreInitializationEvent event) {
        // Forced earlier so it is available to the crystalthorn. Bit jank...
        Config.initConfig(new File(event.getModConfigurationDirectory(), "acronym/floralchemy.cfg"));
        super.preInit(event);
        proxy.registerModels();
    }

    @Override
    @EventHandler
    public void init(FMLInitializationEvent event) {
        super.init(event);
        // Loot
        LootFunctionManager.func_186582_a(new LootFunctionCrystalthorn.Serializer());
        LootTableList.func_186375_a(new ResourceLocation(MOD_ID, "inject/end_city_treasure"));
        LootTableList.func_186375_a(new ResourceLocation(MOD_ID, "block/hedge"));
        LootTableList.func_186375_a(new ResourceLocation(MOD_ID, "block/thorny_hedge"));
        // Vilages
        VillagerRegistry.instance().registerVillageCreationHandler(new VillageHedgedHouseHandler());
        MapGenStructureIO.func_143031_a(VillageHedgeHouse.class, "hedge_house");
    }

    @Override
    @EventHandler
    public void postInit(FMLPostInitializationEvent event) {
        super.postInit(event);
    }

    @Override
    public void registerBlocks(BlockRegistry registry) {
        for(Tuple<ResourceLocation, Integer> entry : FloralchemyAPI.getCrystalRegistry().getCrystals()) {
            Block block = new BlockCrystalthorn(entry);
            registry.register(new ResourceLocation(Floralchemy.MOD_ID, "crystalthorn_" + entry.func_76341_a().func_110623_a()),
                    block);
        }
        registry.register(new BlockHedge("hedge", false));
        registry.register(new BlockHedge("thorny_hedge", true));
        registry.register(new BlockFloodedSoil());
        registry.register(new BlockLeakyCauldron());
    }

    @Override
    public Floralchemy getInstance() {
        return this;
    }

    @Override
    public boolean hasConfig() {
        return false;
    }
}
